//Tonemap Based on Filmic Worlds
//http://filmicworlds.com/

float filmc(float x) {
float A = 0.22;
float B = 0.3;
float C = 0.15 * 1.70;
float D = 0.4 * 1.30;
float E = 0.01 * 1.47;
float F = 0.2;
return ((x * (A * x + C * B) + D * E) / (x * (A * x + B) + D * F)) - E / F;
}

vec3 U2TM(vec3 clr) {
float W = 1.3 / 1.0;
float Luma = dot(clr, vec3(0.0, 0.3, 0.3));
vec3 Chroma = clr - Luma;
clr = (Chroma * 1.70) + Luma;
  clr = vec3(filmc(clr.r), filmc(clr.g), filmc(clr.b)) / filmc(W);
return clr;
}

vec3 saturation(inout vec3 color,float satBoost) {
color.r = color.r * (1.0 + satBoost * 2.0) - (color.g * satBoost) - (color.b * satBoost);
color.g = color.g * (1.0 + satBoost * 2.0) - (color.r * satBoost) - (color.b * satBoost);
color.b = color.b * (1.0 + satBoost * 2.0) - (color.r * satBoost) - (color.g * satBoost);
return color;
}

vec3 grayscale(vec3 tex){
tex /= 3.0;
tex = vec3(tex.r + tex.g + tex.b);
        vec3 gray = vec3(dot(vec3(0.3), tex.rgb));
        vec3 color = gray * (vec3(0.7, 0.9, 1.3) * 0.12);
        return max(gray,color);
}

vec3 czm_exposure(vec3 diffuse)
{
float gamma = 0.0;
float exposure = 1.7;
diffuse *= exposure/(1. + diffuse / exposure);
//.diffuse = pow(diffuse, vec3(1. / gamma));
return diffuse;
}

float specu(vec2 uv,float a,float b){
float specu;
specu = smoothstep(a,b,length(uv));
return specu;
}

vec3 disableAO(vec3 baseColour, vec4 vertexColour) {

    vec3 result = baseColour;

    if (vertexColour.r != vertexColour.g && vertexColour.g != vertexColour.b || vertexColour.r != vertexColour.b && vertexColour.g != vertexColour.b || vertexColour.g < vertexColour.r /* Redstone dust */) {
        if (vertexColour.a != 0.0){
            result *= normalize(vertexColour.rgb);
        }
    }
    if (vertexColour.a == 0.0){
        result *= vertexColour.rgb * 1.35 /* Brightness adjustment for leaves */ ;
    }

    return result; // Completely disable AO (with redstone dust brightness)
}

vec3 jodieReinhardTonemap(vec3 c){
float l = dot(c, vec3(0.2126, 0.7152, 0.0722));
vec3 tc = c / (c + 1.0);

return mix(c / (l + 1.0), tc, tc);
}

vec3 contrastFilter(vec3 color, float contrast) {
float t = 0.5 - contrast * 0.5;

return clamp(color * contrast + t, 0.0, 1.0);
}

float Bloom(vec2 p){
float bloom;
bloom = clamp(1.0-length(p)*0.1,0.0,1.0);
return bloom;
}

float brightMap(vec3 map){
float bright = (map.r+map.g+map.b)/3.0;
return bright;
}

float luminance(vec3 c){
	return dot(c, vec3(0.2126, 0.7152, 0.0722));
}